/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "MysticRealms.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

MysticRealms::MysticRealms() {
    gameState = VD_GAMEWORLD;
    gameRunning = TRUE;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

MysticRealms::~MysticRealms() {
};



/* **********************************************************************************************************************
   Run
   ---
   Starts running the Mystic Realms game.
   #param  instance  Instance of the application.
   **********************************************************************************************************************/

void MysticRealms::Run(HINSTANCE instance) {
    VDWindow window;
    VDDirect3D graphics;
    VDInput input;
    VDWorld world;
    VDEntity player;
    __int64 currentCount;
    MSG message;
    int width, height;
    BOOL enteringNewArea = FALSE;

    // initialize variables
    srand((unsigned int) time(NULL));  // TODO:  move to INIT game state, when we make it
    player.SetCurrentLocation(0, 0);
    width = 640;
    height = 480;

    // create the application window
    window.CreateMyWindow(instance, "Mystic Realms", width, height);

    // initialize the graphics API
    graphics.InitializeDirect3D(window.GetWindow());

    // initialize input
    input.InitializeInput(instance);
    input.InitializeKeyboard(window.GetWindow());

    // load world graphics
    world.LoadWorld("World.vdm");
    for (int x = 0; x < world.GetWorldSizeX(); x++) {
        for (int y = 0; y < world.GetWorldSizeY(); y++) {
            graphics.AddSprite(world.GetTileImageFileLocation(x, y),
                               (float) x * 32.0f,
                               (float) y * 32.0f,
                               32,
                               32,
                               FALSE,
                               FALSE);
        };
    };
    for (unsigned int n = 0; n < world.GetNPCsSize(); n++) {
        graphics.AddSprite(world.GetNPCImageFileLocation(n),
                          (float) world.GetNPCCurrentLocationX(n) * 32.0f,
                          (float) world.GetNPCCurrentLocationY(n) * 32.0f,
                          32,
                          32,
                          FALSE,
                          TRUE);
    };

    graphics.AddSprite("../MysticRealms/images/characters/dg_classm32_10.png", (float) width / 2, (float) height / 2, 32, 32, TRUE, FALSE);

    // high performance timer (TODO: make into a class???)
    currentCount = 0;
    double gameUpdateCount = 0.0f;
    const double frequency = 60.0f;
    const double maxSecondsForOneFrame = 1.0f / frequency;
    __int64 processorFrequency;
    BOOL returnValue = QueryPerformanceFrequency((LARGE_INTEGER *) &processorFrequency);
    if (returnValue == FALSE) {
    };
    returnValue = QueryPerformanceCounter((LARGE_INTEGER *) &currentCount);
    if (returnValue == FALSE) {
    };
    double accumulator = 0.0f;
    double currentTime = 0.0f;

    // game loop
    while (gameRunning) {
        __int64 newCount;
        returnValue = QueryPerformanceCounter((LARGE_INTEGER *) &newCount);
        if (returnValue == FALSE) {
        };

        double deltaTime = (double) (newCount - currentCount) / processorFrequency;
        currentCount = newCount;
        currentTime = (double) currentCount / processorFrequency;

        // clamp
        if (deltaTime > 0.25) {
            deltaTime = 0.25;
        };

        accumulator += deltaTime;

        while (accumulator >= maxSecondsForOneFrame) {
            //TODO update game logic, input, all that jazz.  use (double) currentTime/processorFrequency for the amount of seconds past in the game
            gameUpdateCount += maxSecondsForOneFrame;
            accumulator -= maxSecondsForOneFrame;
        };

        // render

        // dispatch incoming sent messages, check the thread message queue, retrieve message
        if (PeekMessage(&message, window.GetWindow(), 0, 0, PM_REMOVE)) {
            TranslateMessage(&message);
            DispatchMessage(&message);
        } else {
            // 1. check for user input
            // 2. run AI
            // 3. move enemies
            // 4. resolve collisions
            // 5. draw graphics
            // 6. play sounds

            // gather input
            input.RetrieveLostInput();
            input.UpdateInput();

            // Game State:  World
            if (gameState == VD_GAMEWORLD && graphics.IsMoving() == FALSE) {
                short choice = VD_NONE;

                // see if any keys were pressed
                if (input.GetKeyboardInput()[DIK_ESCAPE] & 0x80) {
                    gameRunning = FALSE;
                } else if (input.GetKeyboardInput()[DIK_LEFT] & 0x80) {
                    choice = VD_LEFT;
                } else if (input.GetKeyboardInput()[DIK_RIGHT] & 0x80) {
                    choice = VD_RIGHT;
                } else if (input.GetKeyboardInput()[DIK_UP] & 0x80) {
                    choice = VD_UP;
                } else if (input.GetKeyboardInput()[DIK_DOWN] & 0x80) {
                    choice = VD_DOWN;
                };

                // TEMPORARY - NPC logic
                for (unsigned int n = 0; n < world.GetNPCsSize(); n++) {
                };

                // user decided to move
                if (choice >= 1 && choice <= 4) {
                    // see if the new tile is available
                    int checkNewLocation = world.CheckNewLocation(choice, player.GetCurrentLocationX(), player.GetCurrentLocationY());
                    switch (checkNewLocation) {
                        // null
                        case 0:
                            break;
                        // open
                        case 1:
                            // update the player location
                            player.Move(choice);

                            // let the graphics know we're moving
                            graphics.MovingPlayer(choice);

                            break;
                        // impassable
                        case 2:
                            break;
                        // new area
                        case 3:
                            // let it know we're entering a new area
                            enteringNewArea = TRUE;

                            // update the player location
                            player.Move(choice);

                            // let the graphics know we're moving
                            graphics.MovingPlayer(choice);

                            break;
                    };
                };

                // are we entering a new area?
                if (enteringNewArea == TRUE && graphics.IsMoving() == FALSE) {
                    char newAreaName[FILE_NAME_MAX_SIZE];

                    // get the starting point of the new area
                    int x = player.GetCurrentLocationX();
                    int y = player.GetCurrentLocationY();
                    int newX = world.GetTileNewAreaStartingPointX(x, y);
                    int newY = world.GetTileNewAreaStartingPointY(x, y);

                    // get the name of the new area
                    (void) strcpy_s(newAreaName, world.GetTileNewAreaName(x, y));

                    // clear world data
                    world.ClearWorld();

                    // load the new area
                    world.LoadWorld(newAreaName);
                    player.SetCurrentLocation(newX, newY);
                    enteringNewArea = FALSE;

                    // clear graphics
                    graphics.ClearScreen();

                    // load tile sprites
                    for (int x = 0; x < world.GetWorldSizeX(); x++) {
                        for (int y = 0; y < world.GetWorldSizeY(); y++) {
                            graphics.AddSprite(world.GetTileImageFileLocation(x, y),
                                               (float) x * 32.0f,
                                               (float) y * 32.0f,
                                               32,
                                               32,
                                               FALSE,
                                               FALSE);
                        };
                    };

                    // load NPC sprites
                    for (unsigned int n = 0; n < world.GetNPCsSize(); n++) {
                        graphics.AddSprite(world.GetNPCImageFileLocation(n),
                                          (float) world.GetNPCCurrentLocationX(n) * 32.0f,
                                          (float) world.GetNPCCurrentLocationY(n) * 32.0f,
                                          32,
                                          32,
                                          FALSE,
                                          TRUE);
                    };

                    // load player sprite
                    graphics.AddSprite("../MysticRealms/images/characters/dg_classm32_10.png", (float) width / 2, (float) height / 2, 32, 32, TRUE, FALSE);
                };
            };

            // draw graphics
            RECT drawingArea;
            drawingArea.left = 0;
            drawingArea.top = 0;
            drawingArea.right = width;
            drawingArea.bottom = height;
            graphics.Render(&drawingArea, width, height, player.GetCurrentLocationX(), player.GetCurrentLocationY());
        };
    };
};