/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDMapEditor.h"



/* **********************************************************************************************************************
   Global Function Prototypes
   **********************************************************************************************************************/

LRESULT CALLBACK WindowProcedure(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDMapEditor::VDMapEditor() {
    window = NULL;
    mapNameEditBox = NULL;
    saveMapButton = NULL;
    loadMapButton = NULL;
    clearMapButton = NULL;
    loadSpriteButton = NULL;
    tileSprite = NULL;
    (void) strcpy_s(tileSpriteLocation, "");
    (void) SecureZeroMemory(&tileSpriteDimensions, sizeof(tileSpriteDimensions));
    tileSpawnsListBox = NULL;
    creaturesListBox = NULL;
    removeSpawnButton = NULL;
    addSpawnButton = NULL;
    nullTileOption = NULL;
    openTileOption = NULL;
    impassableTileOption = NULL;
    newAreaTileOption = NULL;
    loadNewAreaButton = NULL;
    newAreaStatic = NULL;
    (void) strcpy_s(newAreaLocation, "");
    newAreaStartingPointX = NULL;
    newAreaStartingPointY = NULL;
    npcsListBox = NULL;
    npcSprite = NULL;
    (void) strcpy_s(npcSpriteLocation, "");
    (void) SecureZeroMemory(&npcSpriteDimensions, sizeof(npcSpriteDimensions));
    nullNPCOption = NULL;
    normalNPCOption = NULL;
    staticNPCOption = NULL;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDMapEditor::~VDMapEditor() {
    if (tileSprite != NULL) {
        delete tileSprite;
        tileSprite = NULL;
    };
};



/* **********************************************************************************************************************
   CreateMapEditor
   ---------------
   Creates a new Map Editor window.
   #param  instance      The instance of the application.
   #param  parentWindow  Handle to the main window (to use for the starting x, y position)
   #param  className     The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDMapEditor::CreateMapEditor(HINSTANCE instance, HWND parentWindow, LPCSTR className) {
    RECT windowDimensions;
    (void) GetWindowRect(parentWindow, &windowDimensions);
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "Map Editor",                          // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            windowDimensions.left + 100,           // x coordinate (0 = left)
                            windowDimensions.top + 100,            // y coordinate (0 = top)
                            277,                                   // width
                            550,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        VDError("Failed to create the map editor window.", __FILE__, __LINE__);
    };

    // map
    HWND mapNameStatic = CreateWindow("Static", "Map Name:", WS_CHILD | WS_VISIBLE, 10, 10, 80, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_MAP_NAME, instance, NULL);
    if (mapNameStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    mapNameEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 90, 10, 170, 18, window, (HMENU) VD_MAP_EDITOR_MAP_NAME, instance, NULL);
    if (mapNameEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveMapButton = CreateWindow("Button", "Save Map", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 36, 75, 20, window, (HMENU) VD_MAP_EDITOR_SAVE_MAP, instance, NULL);
    if (saveMapButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadMapButton = CreateWindow("Button", "Load Map", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 97, 36, 75, 20, window, (HMENU) VD_MAP_EDITOR_LOAD_MAP, instance, NULL);
    if (loadMapButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    clearMapButton = CreateWindow("Button", "Clear Map", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 185, 36, 75, 20, window, (HMENU) VD_MAP_EDITOR_CLEAR_MAP, instance, NULL);
    if (clearMapButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND divider = CreateWindow("Static", "----------------------------------------------------------------------", WS_CHILD | WS_VISIBLE, 10, 56, 250, 10, window, (HMENU) VD_MAP_EDITOR_DIVIDER, instance, NULL);
    if (divider == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // add tile
    HWND addTileStatic = CreateWindow("Static", "[Add Tile]", WS_CHILD | WS_VISIBLE, 10, 70, 90, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_ADD_TILE, instance, NULL);
    if (addTileStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("Button", "Load Sprite", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 95, 90, 20, window, (HMENU) VD_MAP_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    if (tileSprite != NULL) {
        delete tileSprite;
        tileSprite = NULL;
    };
    (void) strcpy_s(tileSpriteLocation, "");
    tileSpriteDimensions.left = 112;
    tileSpriteDimensions.top = 95;
    tileSpriteDimensions.right = 144;
    tileSpriteDimensions.bottom = 127;
    HWND tileSpawnsStatic = CreateWindow("Static", "Tile Spawns:", WS_CHILD | WS_VISIBLE, 10, 130, 100, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_TILE_SPAWNS, instance, NULL);
    if (tileSpawnsStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    tileSpawnsListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 150, 120, 100, window, (HMENU) VD_MAP_EDITOR_TILE_SPAWNS, instance, NULL);
    if (tileSpawnsListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND creaturesStatic = CreateWindow("Static", "Creature:", WS_CHILD | WS_VISIBLE, 140, 130, 80, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_CREATURES, instance, NULL);
    if (creaturesStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    creaturesListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 140, 150, 120, 100, window, (HMENU) VD_MAP_EDITOR_CREATURES, instance, NULL);
    if (creaturesListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeSpawnButton = CreateWindow("Button", "Remove Spawn", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 255, 110, 20, window, (HMENU) VD_MAP_EDITOR_REMOVE_SPAWN, instance, NULL);
    if (removeSpawnButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    addSpawnButton = CreateWindow("Button", "Add Spawn", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 140, 255, 80, 20, window, (HMENU) VD_MAP_EDITOR_ADD_SPAWN, instance, NULL);
    if (addSpawnButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND tileTypesStatic = CreateWindow("Static", "Tile Types:", WS_CHILD | WS_VISIBLE, 10, 280, 80, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_TILE_TYPES, instance, NULL);
    if (tileTypesStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nullTileOption = CreateWindow("Button", "Null", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON | WS_GROUP, 10, 300, 50, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_NULL, instance, NULL);
    if (nullTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) SendMessage(nullTileOption, BM_SETCHECK, BST_CHECKED, 0);
    openTileOption = CreateWindow("Button", "Open", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 320, 60, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_OPEN, instance, NULL);
    if (openTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    impassableTileOption = CreateWindow("Button", "Impassable", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 340, 100, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_IMPASSABLE, instance, NULL);
    if (impassableTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaTileOption = CreateWindow("Button", "New Area", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 360, 80, 18, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_NEW_AREA, instance, NULL);
    if (newAreaTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadNewAreaButton = CreateWindow("Button", "Load New Area", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 140, 280, 110, 20, window, (HMENU) VD_MAP_EDITOR_LOAD_NEW_AREA, instance, NULL);
    if (loadNewAreaButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaStatic = CreateWindow("Static", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 140, 305, 120, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_NEW_AREA, instance, NULL);
    if (newAreaStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) strcpy_s(newAreaLocation, "");
    HWND newAreaStartingPointStatic = CreateWindow("Static", "New Area\nStarting Point:", WS_CHILD | WS_VISIBLE, 140, 325, 120, 35, window, (HMENU) VD_MAP_EDITOR_STATIC_STARTING_POINT, instance, NULL);
    if (newAreaStartingPointStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaStartingPointX = CreateWindow("Edit", "0", WS_CHILD | WS_VISIBLE | WS_BORDER, 140, 360, 30, 18, window, (HMENU) VD_MAP_EDITOR_STARTING_POINT_X, instance, NULL);
    if (newAreaStartingPointX == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaStartingPointY = CreateWindow("Edit", "0", WS_CHILD | WS_VISIBLE | WS_BORDER, 180, 360, 30, 18, window, (HMENU) VD_MAP_EDITOR_STARTING_POINT_Y, instance, NULL);
    if (newAreaStartingPointY == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND divider2 = CreateWindow("Static", "----------------------------------------------------------------------", WS_CHILD | WS_VISIBLE, 10, 380, 250, 10, window, (HMENU) VD_MAP_EDITOR_DIVIDER2, instance, NULL);
    if (divider2 == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // add npc
    HWND addNPCStatic = CreateWindow("Static", "[Add NPC's]", WS_CHILD | WS_VISIBLE, 10, 395, 80, 18, window, (HMENU) VD_MAP_EDITOR_STATIC_ADD_NPC, instance, NULL);
    if (addNPCStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcsListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 420, 120, 100, window, (HMENU) VD_MAP_EDITOR_NPCS, instance, NULL);
    if (npcsListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
    (void) strcpy_s(npcSpriteLocation, "");
    npcSpriteDimensions.left = 140;
    npcSpriteDimensions.top = 420;
    npcSpriteDimensions.right = 172;
    npcSpriteDimensions.bottom = 452;
    nullNPCOption = CreateWindow("Button", "Null NPC", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 140, 460, 110, 18, window, (HMENU) VD_MAP_EDITOR_NPC_OPTION_NULL, instance, NULL);
    if (nullNPCOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    normalNPCOption = CreateWindow("Button", "Normal NPC", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 140, 480, 110, 18, window, (HMENU) VD_MAP_EDITOR_NPC_OPTION_NORMAL, instance, NULL);
    if (normalNPCOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    staticNPCOption = CreateWindow("Button", "Static NPC", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 140, 500, 100, 18, window, (HMENU) VD_MAP_EDITOR_NPC_OPTION_STATIC, instance, NULL);
    if (staticNPCOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    return window;
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

HWND VDMapEditor::GetWindow() {
    return window;
};

HWND VDMapEditor::GetMapNameEditBox() {
    return mapNameEditBox;
};

HWND VDMapEditor::GetSaveMapButton() {
    return saveMapButton;
};

HWND VDMapEditor::GetLoadMapButton() {
    return loadMapButton;
};

HWND VDMapEditor::GetClearMapButton() {
    return clearMapButton;
};

HWND VDMapEditor::GetLoadSpriteButton() {
    return loadSpriteButton;
};

VDSprite* VDMapEditor::GetTileSprite() {
    return tileSprite;
};

void VDMapEditor::SetTileSprite(VDSprite *newSprite) {
    if (tileSprite != NULL) {
        delete tileSprite;
        tileSprite = NULL;
    };
    tileSprite = newSprite;
};

LPCSTR VDMapEditor::GetTileSpriteLocation() {
    return tileSpriteLocation;
};

void VDMapEditor::SetTileSpriteLocation(LPCSTR spriteLocation) {
    (void) strcpy_s(tileSpriteLocation, spriteLocation);
};

LPRECT VDMapEditor::GetTileSpriteDimensions(LPRECT outRect) {
    outRect->left = tileSpriteDimensions.left;
    outRect->top = tileSpriteDimensions.top;
    outRect->right = tileSpriteDimensions.right;
    outRect->bottom = tileSpriteDimensions.bottom;

    return &tileSpriteDimensions;
};

HWND VDMapEditor::GetTileSpawnsListBox() {
    return tileSpawnsListBox;
};

HWND VDMapEditor::GetCreaturesListBox() {
    return creaturesListBox;
};

HWND VDMapEditor::GetRemoveSpawnButton() {
    return removeSpawnButton;
};

HWND VDMapEditor::GetAddSpawnButton() {
    return addSpawnButton;
};

HWND VDMapEditor::GetNullTileOption() {
    return nullTileOption;
};

HWND VDMapEditor::GetOpenTileOption() {
    return openTileOption;
};

HWND VDMapEditor::GetImpassableTileOption() {
    return impassableTileOption;
};

HWND VDMapEditor::GetNewAreaTileOption() {
    return newAreaTileOption;
};

HWND VDMapEditor::GetLoadNewAreaButton() {
    return loadNewAreaButton;
};

HWND VDMapEditor::GetNewAreaStatic() {
    return newAreaStatic;
};

LPCSTR VDMapEditor::GetNewAreaLocation() {
    return newAreaLocation;
};

void VDMapEditor::SetNewAreaLocation(LPCSTR aNewAreaLocation) {
    (void) strcpy_s(newAreaLocation, aNewAreaLocation);
};

HWND VDMapEditor::GetNewAreaStartingPointX() {
    return newAreaStartingPointX;
};

HWND VDMapEditor::GetNewAreaStartingPointY() {
    return newAreaStartingPointY;
};

HWND VDMapEditor::GetNPCListBox() {
    return npcsListBox;
};

VDSprite* VDMapEditor::GetNPCSprite() {
    return npcSprite;
};

void VDMapEditor::SetNPCSprite(VDSprite *newSprite) {
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
    npcSprite = newSprite;
};

LPCSTR VDMapEditor::GetNPCSpriteLocation() {
    return npcSpriteLocation;
};

void VDMapEditor::SetNPCSpriteLocation(LPCSTR spriteLocation) {
    (void) strcpy_s(npcSpriteLocation, spriteLocation);
};

LPRECT VDMapEditor::GetNPCSpriteDimensions(LPRECT outRect) {
    outRect->left = npcSpriteDimensions.left;
    outRect->top = npcSpriteDimensions.top;
    outRect->right = npcSpriteDimensions.right;
    outRect->bottom = npcSpriteDimensions.bottom;

    return &npcSpriteDimensions;
};

HWND VDMapEditor::GetNullNPCOption() {
    return nullNPCOption;
};

HWND VDMapEditor::GetNormalNPCOption() {
    return normalNPCOption;
};

HWND VDMapEditor::GetStaticNPCOption() {
    return staticNPCOption;
};