/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDNPCEditor.h"



/* **********************************************************************************************************************
   Global Function Prototypes
   **********************************************************************************************************************/

LRESULT CALLBACK WindowProcedure(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDNPCEditor::VDNPCEditor() {
    window = NULL;
    npcListBox = NULL;
    nameEditBox = NULL;
    loadSpriteButton = NULL;
    npcSprite = NULL;
    (void) strcpy_s(npcSpriteLocation, "");
    (void) SecureZeroMemory(&npcSpriteDimensions, sizeof(npcSpriteDimensions));
    removeNPCButton = NULL;
    saveNPCButton = NULL;
    dialogueListBox = NULL;
    textEditBox = NULL;
    textIDEditBox = NULL;
    removeDialogueButton = NULL;
    saveDialogueButton = NULL;
    optionListBox = NULL;
    textEditBox2 = NULL;
    textIDEditBox2 = NULL;
    removeOptionButton = NULL;
    saveOptionButton = NULL;
    saveChangesButton = NULL;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDNPCEditor::~VDNPCEditor() {
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
};



/* **********************************************************************************************************************
   CreateNPCEditor
   ---------------
   Creates a new NPC Editor window.
   #param  instance      The instance of the application.
   #param  parentWindow  Handle to the main window (to use for the starting x, y position)
   #param  className     The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDNPCEditor::CreateNPCEditor(HINSTANCE instance, HWND parentWindow, LPCSTR className) {
    RECT windowDimensions;
    (void) GetWindowRect(parentWindow, &windowDimensions);
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "NPC Editor",                          // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            windowDimensions.left + 100,           // x coordinate (0 = left)
                            windowDimensions.top + 100,            // y coordinate (0 = top)
                            440,                                   // width
                            530,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        VDError("Failed to create the window.", __FILE__, __LINE__);
    };

    // NPC's
    HWND npcListStatic = CreateWindow("Static", "NPC's:", WS_CHILD | WS_VISIBLE, 10, 10, 40, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NPC_LIST, instance, NULL);
    if (npcListStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcListBox = CreateWindow("ListBox", NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 30, 100, 100, window, (HMENU) VD_NPC_EDITOR_NPC_LIST, instance, NULL);
    if (npcListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND npcNameStatic = CreateWindow("Static", "Name:", WS_CHILD | WS_VISIBLE, 120, 30, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NAME, instance, NULL);
    if (npcNameStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nameEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 180, 30, 140, 18, window, (HMENU) VD_NPC_EDITOR_NAME, instance, NULL);
    if (nameEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("Button", "Load Sprite", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 120, 75, 90, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
    (void) strcpy_s(npcSpriteLocation, "");
    npcSpriteDimensions.left = 225;
    npcSpriteDimensions.top = 63;
    npcSpriteDimensions.right = 257;
    npcSpriteDimensions.bottom = 95;
    removeNPCButton = CreateWindow("Button", "Remove NPC", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 132, 100, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (removeNPCButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveNPCButton = CreateWindow("Button", "Save NPC", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 120, 107, 80, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (saveNPCButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // NPC dialogue
    HWND dialogueStatic = CreateWindow("Static", "Dialogues", WS_CHILD | WS_VISIBLE, 10, 160, 70, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_DIALOGUE, instance, NULL);
    if (dialogueStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    dialogueListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 180, 200, 110, window, (HMENU) VD_NPC_EDITOR_DIALOGUE, instance, NULL);
    if (dialogueListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textStatic = CreateWindow("Static", "Text:", WS_CHILD | WS_VISIBLE, 220, 160, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT, instance, NULL);
    if (textStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textEditBox = CreateWindow("Edit", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE, 220, 180, 200, 60, window, (HMENU) VD_NPC_EDITOR_TEXT, instance, NULL);
    if (textEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textIDStatic = CreateWindow("Static", "Text ID:", WS_CHILD | WS_VISIBLE, 220, 250, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT_ID, instance, NULL);
    if (textIDStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textIDEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 275, 250, 30, 18, window, (HMENU) VD_NPC_EDITOR_TEXT_ID, instance, NULL);
    if (textIDEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeDialogueButton = CreateWindow("Button", "Remove Dialogue", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 283, 130, 20, window, (HMENU) VD_NPC_EDITOR_REMOVE_DIALOGUE, instance, NULL);
    if (removeDialogueButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveDialogueButton = CreateWindow("Button", "Save Text", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 320, 250, 80, 20, window, (HMENU) VD_NPC_EDITOR_SAVE_DIALOGUE, instance, NULL);
    if (saveDialogueButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // options
    HWND optionsStatic = CreateWindow("Static", "User Response Options", WS_CHILD | WS_VISIBLE, 10, 310, 200, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_OPTIONS, instance, NULL);
    if (optionsStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    optionListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 330, 200, 110, window, (HMENU) VD_NPC_EDITOR_OPTIONS, instance, NULL);
    if (optionListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textStatic2 = CreateWindow("Static", "Text:", WS_CHILD | WS_VISIBLE, 220, 310, 70, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT2, instance, NULL);
    if (textStatic2 == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textEditBox2 = CreateWindow("Edit", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE, 220, 330, 200, 60, window, (HMENU) VD_NPC_EDITOR_TEXT2, instance, NULL);
    if (textEditBox2 == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textIDStatic2 = CreateWindow("Static", "Next ID:", WS_CHILD | WS_VISIBLE, 220, 400, 55, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT_ID2, instance, NULL);
    if (textIDStatic2 == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textIDEditBox2 = CreateWindow("Edit", "", WS_CHILD | WS_VISIBLE | WS_BORDER, 278, 400, 30, 18, window, (HMENU) VD_NPC_EDITOR_TEXT_ID2, instance, NULL);
    if (textIDEditBox2 == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeOptionButton = CreateWindow("Button", "Remove Response", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 432, 140, 20, window, (HMENU) VD_NPC_EDITOR_REMOVE_OPTION, instance, NULL);
    if (removeOptionButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveOptionButton = CreateWindow("Button", "Save Text", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 320, 400, 80, 20, window, (HMENU) VD_NPC_EDITOR_SAVE_OPTION, instance, NULL);
    if (saveOptionButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // save NPC's
    saveChangesButton = CreateWindow("Button", "SAVE CHANGES", WS_CHILD | BS_PUSHBUTTON | WS_BORDER | WS_VISIBLE, 115, 460, 200, 40, window, (HMENU) VD_NPC_EDITOR_SAVE_CHANGES, instance, NULL);
    if (saveChangesButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    return window;
};








/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

HWND VDNPCEditor::GetWindow() {
    return window;
};

HWND VDNPCEditor::GetNPCListBox() {
    return npcListBox;
};

HWND VDNPCEditor::GetNameEditBox() {
    return nameEditBox;
};

HWND VDNPCEditor::GetLoadSpriteButton() {
    return loadSpriteButton;
};

VDSprite* VDNPCEditor::GetNPCSprite() {
    return npcSprite;
};

void VDNPCEditor::SetNPCSprite(VDSprite *newSprite) {
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
    npcSprite = newSprite;
};

LPCSTR VDNPCEditor::GetNPCSpriteLocation() {
    return npcSpriteLocation;
};

void VDNPCEditor::SetNPCSpriteLocation(LPCSTR spriteLocation) {
    (void) strcpy_s(npcSpriteLocation, spriteLocation);
};

LPRECT VDNPCEditor::GetNPCSpriteDimensions(LPRECT outRect) {
    outRect->left = npcSpriteDimensions.left;
    outRect->top = npcSpriteDimensions.top;
    outRect->right = npcSpriteDimensions.right;
    outRect->bottom = npcSpriteDimensions.bottom;

    return &npcSpriteDimensions;
};

HWND VDNPCEditor::GetRemoveNPCButton() {
    return removeNPCButton;
};

HWND VDNPCEditor::GetSaveNPCButton() {
    return saveNPCButton;
};

HWND VDNPCEditor::GetDialogueListBox() {
    return dialogueListBox;
};

HWND VDNPCEditor::GetTextEditBox() {
    return textEditBox;
};

HWND VDNPCEditor::GetTextIDEditBox() {
    return textIDEditBox;
};

HWND VDNPCEditor::GetRemoveDialogueButton() {
    return removeDialogueButton;
};

HWND VDNPCEditor::GetSaveDialogueButton() {
    return saveDialogueButton;
};

HWND VDNPCEditor::GetOptionsListBox() {
    return optionListBox;
};

HWND VDNPCEditor::GetTextEditBox2() {
    return textEditBox2;
};

HWND VDNPCEditor::GetTextIDEditBox2() {
    return textIDEditBox2;
};

HWND VDNPCEditor::GetRemoveOptionButton() {
    return removeOptionButton;
};

HWND VDNPCEditor::GetSaveOptionButton() {
    return saveOptionButton;
};

HWND VDNPCEditor::GetSaveChangesButton() {
    return saveChangesButton;
};